SystemOrganization addCategory: #'AL-User'!

Notification subclass: #ALAuthenticationRequest
	instanceVariableNames: 'username password'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALAuthenticationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/24/2012 09:51'!
username: username password: password
	^ (self withUsername: username password: password) signal! !

!ALAuthenticationRequest class methodsFor: 'as yet unclassified' stamp: 'cwp 12/24/2012 09:20'!
withUsername: username password: password
	^ self basicNew
		initializeWithUsername: username password: password! !

!ALAuthenticationRequest methodsFor: 'initialization' stamp: 'cwp 12/24/2012 09:52'!
initializeWithUsername: aUsername password: aPassword
	self initialize.
	username := aUsername.
	password := aPassword.! !

!ALAuthenticationRequest methodsFor: 'interpreting' stamp: 'cwp 12/25/2012 11:30'!
name: aString value: anObject
	aString = 'username' ifTrue: [username := anObject].
	aString = 'password' ifTrue: [password := anObject].! !

!ALAuthenticationRequest methodsFor: 'interpreting' stamp: 'cwp 12/25/2012 11:30'!
object: aBlock
	aBlock value
	! !

!ALAuthenticationRequest methodsFor: 'accessing' stamp: 'cwp 12/24/2012 09:52'!
password
	^ password! !

!ALAuthenticationRequest methodsFor: 'accessing' stamp: 'cwp 12/24/2012 09:52'!
username
	^ username! !

Notification subclass: #ALUserStipulation
	instanceVariableNames: 'user'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALUserStipulation class methodsFor: 'as yet unclassified' stamp: 'cwp 1/8/2013 21:55'!
user: aUser
	^ (self withUser: aUser) signal! !

!ALUserStipulation class methodsFor: 'as yet unclassified' stamp: 'cwp 1/8/2013 21:54'!
withUser: aUser
	^ self basicNew initializeWithUser: aUser! !

!ALUserStipulation methodsFor: 'as yet unclassified' stamp: 'cwp 1/8/2013 21:53'!
initializeWithUser: aUser
	self initialize.
	user := aUser! !

!ALUserStipulation methodsFor: 'as yet unclassified' stamp: 'cwp 1/8/2013 21:53'!
user
	^ user! !

ALRelay subclass: #ALAuthenticationRelay
	instanceVariableNames: 'authenticator shunt'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALAuthenticationRelay class methodsFor: 'instance creation' stamp: 'cwp 12/26/2012 09:24'!
destination: aRelay authenticator: anAuthenticator shunt: aBlock
	^ self basicNew 
		initializeWithDestination: aRelay
		authenticator: anAuthenticator
		shunt: aBlock! !

!ALAuthenticationRelay methodsFor: 'authenticating' stamp: 'cwp 12/23/2012 00:16'!
handleAuthRequest: aNotification
	| result |
	result := authenticator 
		authenticate: aNotification username 
		password: aNotification password.
	aNotification resume: result! !

!ALAuthenticationRelay methodsFor: 'relaying' stamp: 'cwp 12/22/2012 23:59'!
handleRequest: aRequest
	^ [destination handleRequest: aRequest]
		on: ALUserStipulation, ALAuthenticationRequest
		do: [:notification |
			notification class == ALUserStipulation
				ifTrue: [self validate: aRequest against: notification]
				ifFalse: [self handleAuthRequest: notification]]! !

!ALAuthenticationRelay methodsFor: 'initialize-release' stamp: 'cwp 12/26/2012 09:25'!
initializeWithDestination: aRelay authenticator: anAuthenticator shunt: aBlock
	self initializeWithDestination: aRelay.
	authenticator := anAuthenticator.
	shunt := aBlock! !

!ALAuthenticationRelay methodsFor: 'authenticating' stamp: 'cwp 1/8/2013 21:57'!
validate: aRequest against: aStipulation
	| cookie doShunt |
	doShunt := [^ shunt value: aRequest].
	aRequest hasCookie ifFalse: doShunt.
	cookie := aRequest cookie 
		detect: [:ea | ea name = 'auth'] 
		ifNone: doShunt.
	(authenticator isToken: cookie value validFor: aStipulation user)
		ifTrue: [aStipulation resume]
		ifFalse: doShunt! !

Object subclass: #ALAuthenticationResult
	instanceVariableNames: 'user token'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALAuthenticationResult class methodsFor: 'instance creation' stamp: 'cwp 12/24/2012 10:12'!
failed
	^ self user: nil token: nil! !

!ALAuthenticationResult class methodsFor: 'instance creation' stamp: 'cwp 12/24/2012 10:11'!
user: aKey token: aString
	^ self basicNew
		initializeWithUser: aKey token: aString! !

!ALAuthenticationResult methodsFor: 'initialization' stamp: 'cwp 12/24/2012 10:12'!
initializeWithUser: aKey token: aString
	self initialize.
	user := aKey.
	token := aString.! !

!ALAuthenticationResult methodsFor: 'testing' stamp: 'cwp 12/24/2012 10:10'!
isSuccess
	^ token notNil! !

!ALAuthenticationResult methodsFor: 'accessing' stamp: 'cwp 12/24/2012 10:10'!
token
	^ token! !

!ALAuthenticationResult methodsFor: 'accessing' stamp: 'cwp 12/24/2012 10:10'!
user
	^ user! !

Object subclass: #ALAuthenticator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALAuthenticator methodsFor: 'public' stamp: 'cwp 12/23/2012 00:02'!
authenticate: username password: password
	self subclassResponsibility! !

!ALAuthenticator methodsFor: 'public' stamp: 'cwp 12/23/2012 00:00'!
isToken: aString validFor: aUserId
	self subclassResponsibility! !

!ALAuthenticator methodsFor: 'public' stamp: 'cwp 1/8/2013 21:56'!
register: aUser username: username password: password
	self subclassResponsibility! !

ALAuthenticator subclass: #ALMemoryAuthenticator
	instanceVariableNames: 'users passwords tokens'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALMemoryAuthenticator methodsFor: 'public' stamp: 'cwp 12/23/2012 00:12'!
authenticate: username password: password
	| fail |
	fail := [ ^ self fail ].
	(passwords at: username ifAbsent: fail) = password
		ifTrue: [ ^ self authenticated: username ]
		ifFalse: fail! !

!ALMemoryAuthenticator methodsFor: 'private' stamp: 'cwp 1/8/2013 21:56'!
authenticated: username
	| token |
	token := self generateToken.
	tokens at: (users at: username) put: token.
	^ ALAuthenticationResult user: (users at: username) token: token! !

!ALMemoryAuthenticator methodsFor: 'private' stamp: 'cwp 12/23/2012 00:11'!
fail
	^ ALAuthenticationResult failed! !

!ALMemoryAuthenticator methodsFor: 'private' stamp: 'cwp 12/23/2012 00:10'!
generateToken
	^ UUID new printString! !

!ALMemoryAuthenticator methodsFor: 'initialize-release' stamp: 'cwp 1/8/2013 21:56'!
initialize
	users := Dictionary new.
	passwords := Dictionary new.
	tokens := Dictionary new.! !

!ALMemoryAuthenticator methodsFor: 'public' stamp: 'cwp 1/8/2013 21:51'!
isToken: aString validFor: aUser
	| token |
	token := tokens at: aUser ifAbsent: [^ false].
	^ token = aString! !

!ALMemoryAuthenticator methodsFor: 'public' stamp: 'cwp 1/8/2013 21:57'!
register: aUser username: username password: password
	users at: username put: aUser.
	passwords at: username put: password! !

ALResource subclass: #ALAuthenticationResource
	instanceVariableNames: 'caller'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-User'!

!ALAuthenticationResource class methodsFor: 'as yet unclassified' stamp: 'cwp 12/24/2012 08:25'!
component: aComponent
	^ self basicNew initializeWithComponent: aComponent! !

!ALAuthenticationResource methodsFor: 'as yet unclassified' stamp: 'cwp 1/8/2013 21:08'!
authorized: result locator: aLocator
	| component cookie url |
	component := caller rematerialize.
	component user: result user.
	url := aLocator urlForResource: component asResource.
	cookie := ALCookie name: 'auth' value: result token.
	cookie path: '/'.
	^ ALResponse created
		location: url;
		setCookie: cookie;
		yourself! !

!ALAuthenticationResource methodsFor: 'as yet unclassified' stamp: 'cwp 12/25/2012 11:20'!
denied
	^ ALResponse status: 401 reason: 'Authentication failed'! !

!ALAuthenticationResource methodsFor: 'as yet unclassified' stamp: 'cwp 12/25/2012 11:37'!
handlePost: aRequest locator: aLocator
	| request result |
	request := ALAuthenticationRequest new.
	aRequest readEntityWith: 
		[:in | ALJsonParser  parse: in for: request].
	result := request signal.
	^ (result notNil and: [result isSuccess])
		ifTrue: [self authorized: result locator: aLocator]
		ifFalse: [self denied]! !

!ALAuthenticationResource methodsFor: 'as yet unclassified' stamp: 'cwp 12/25/2012 11:40'!
initializeWithComponent: aComponent
	self initialize.
	caller := aComponent.
	! !
